(*
 *   An unit file that implement S25R system
 *   Copyright (C) 2005 Fumio Kawamata
 *   Created date: October 15, 2005
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.
 *
 *                      *********************************
 *
 *    This unit just implements "The Selective SMTP Rejection (S25R) System"
 *    by ASAMI Hideo, Japan.
 *)


unit S25R;
(* This unit requires lookup unit *)

interface

uses
  Classes;

var
  RuleList: TStringList;


  procedure CheckRulesByIPAddress(DNSServer, CheckIP: String; var HostName: String; var ReturnedIP: String; var Result: Integer);
  (* Check S25R Rules
   * Result
   *   Returns 1..n when the HostName matched the S25R rule 1..n
   *   Returns 0 when the HostName is not matched any S25R rule
   *   Returns -1 when the reverse lookup failure is occured
   *   Returns -2 when the forward lookup failure is occured
   *   Returns -3 when two IP addresses are different
   *)


  procedure CreateThem;
  (* Create the Rules TStringList *)


  procedure FreeThem;
  (* Free the Rules TStringList *)

  function LoadRules(RuleFileName: String): Boolean;
  (* Returns True  when it is successfully finished, and
   * returns False when it is failed.
   *)



implementation

uses
  SysUtils, lookup,
  BregExp;
  (* BregExp.pas Copyright (C) Osamu Takeuchi All rights Reserved. *)
  (* BregExp.pas includes Japanese. *)
  (* BregExp.pas requires BregExp.dll, Copyright (C) Tatsuo Baba *)


procedure CreateThem;
begin
  RuleList:=TStringList.Create;
  RuleList.Clear;
  lookup.CreateComponents;
end;

procedure FreeThem;
begin
  RuleList.Free;
  lookup.FreeComponents;
end;


function LoadRules(RuleFileName: String): Boolean;
var
  F: text;
  S: String;
begin
  RuleList.Clear;
  Result:=True;
  try
    AssignFile(F, RuleFileName);
    Reset(F);
    while Eof(F)=False do
    begin
      Readln(F, S);
      RuleList.Add(S);
    end;
  except
    CloseFile(F);
    Result:=False;
  end;
end;




function CheckRulesByHostName(HostName: String): Integer;
var
  I: Integer;
  Matched: Boolean;
begin
  Result:=0;
  if RuleList.Count=0 then
  begin
    exit;
  end;
  I:=0;
  Matched:=False;
  while (I<=RuleList.Count-1)and(Matched=False) do
  begin
    Matched:=brx.Match(RuleList[I], HostName);
    inc(I);
  end;
  if Matched=True then
    Result:=I;
end;



procedure CheckRulesByIPAddress(DNSServer, CheckIP: String; var HostName: String; var ReturnedIP: String; var Result: Integer);
  (* Check S25R Rules
   *   Returns 1..n when the HostName matched the S25R rule 1..n
   *   Returns 0 when the HostName is not matched any S25R rule
   *   Returns -1 when the reverse lookup failure is occured
   *   Returns -2 when the forward lookup failure is occured
   *   Returns -3 when two IP addresses are different
   *)
var
  I: Integer;
  IPList: TStrings;
  Matched: Boolean;
begin
  (* Step1: Obtain HostName from CheckIP *)
  HostName:=IpToHostName(CheckIP);
  if HostName='' then
  begin
    Result:=-1;
    exit;
  end;

  (* Step2: Obtain forward lookuped IP addresses *)
  IPList:=TStringList.Create;
  lookup.HostNameToIPAddresses(DNSServer, HostName, IPList);
  if IPList.Count=0 then
  begin (* Number of IP addresses is zero *)
    IPList.Free;
    Result:=-2;
    exit;
  end
  else
  begin (* Number of IP addresses is greater than zero *)
    I:=0;
    ReturnedIP:='';
    Matched:=False;
    while (I<=IPList.Count-1)and(Matched=False) do
    begin
      if CheckIP=IPList[I] then
      begin
        Matched:=True;
        ReturnedIP:=CheckIP;
      end;
      inc(I);
    end;
    if Matched=False then
    begin
      ReturnedIP:=IPList[0]; (* Returns first IP address *)
      Result:=-3;
    end;
  end;
  IPList.Free;

  (* Step3: Check S25R Rules *)
  Result:=CheckRulesByHostName(HostName);
end;



end.
